""".. module:: __init__

.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""

from sitri.configurator import SitriProviderConfigurator

Sitri = SitriProviderConfigurator  # noqa


__version__ = "2.2.11"
