"""
.. module:: providers.contrib.vault.__init__
   :synopsis: Contrib additional functional with vault
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""

from sitri.providers.contrib.vault.vault_kv import VaultKVConfigProvider
